import re
from datetime import date
from django import forms
from django.core.exceptions import ValidationError
from django.utils.html import strip_tags
from .models import UserDanantaro
from django.contrib.auth.forms import AuthenticationForm, UserCreationForm
from django.contrib.auth.models import User

class UserForm(UserCreationForm):
    name = forms.CharField(max_length=255, required=True, widget=forms.TextInput(attrs={'placeholder': 'Masukkan nama Anda'}))
    birth_date = forms.DateField(required=True, widget=forms.DateInput(attrs={'type': 'date'}))
    phone = forms.CharField(max_length=15, required=True, widget=forms.TextInput(attrs={'placeholder': 'Masukkan nomor HP'}))
    url_blog = forms.URLField(required=False, widget=forms.URLInput(attrs={'placeholder': 'Masukkan URL blog'}))
    description = forms.CharField(widget=forms.Textarea(attrs={'placeholder': 'Masukkan deskripsi'}), required=False)
    no_rekening = forms.CharField(max_length=10, required=True, widget=forms.TextInput(attrs={'placeholder': 'Masukkan nomor rekening'}))
    credit_score = forms.IntegerField(required=True, widget=forms.NumberInput(attrs={'placeholder': 'Masukkan skor kredit'}))
    no_card = forms.CharField(max_length=16, required=True, widget=forms.TextInput(attrs={'placeholder': 'Masukkan nomor kartu ATM'}))
    employee_status = forms.ChoiceField(choices=[('', '-Pilih Status Pekerjaan-')] + UserDanantaro.EMPLOYEE_STATUS_CHOICES, required=True)
    code_currency = forms.CharField(max_length=3, required=True, widget=forms.TextInput(attrs={'placeholder': 'Masukkan kode mata uang'}))

    class Meta:
        model = User
        fields = ['username', 'email', 'password1', 'password2']  # Field dari model User

    def clean_birth_date(self):
        birth_date = self.cleaned_data.get('birth_date')
        today = date.today()
        min_age = 12
        if birth_date and (today - birth_date).days < 365 * min_age:
            raise forms.ValidationError('Usia pengguna minimal 12 tahun')
        return birth_date

    def clean_phone(self):
        phone = self.cleaned_data.get('phone')
        if not re.match(r'^\d{8,15}$', phone):
            raise forms.ValidationError('Nomor HP harus terdiri dari 8 hingga 15 digit angka tanpa simbol "+"')
        return phone

    def clean_no_rekening(self):
        no_rekening = self.cleaned_data.get('no_rekening')
        if not re.match(r'^\d{8,10}$', no_rekening):
            raise forms.ValidationError('Nomor rekening harus terdiri dari 8 hingga 10 digit angka.')
        return no_rekening

    def clean_no_card(self):
        no_card = self.cleaned_data.get('no_card')
        if not re.match(r'^\d{15,16}$', no_card):
            raise forms.ValidationError('Nomor kartu ATM harus terdiri dari 15 atau 16 digit angka.')
        return no_card

    def clean_code_currency(self):
        code_currency = self.cleaned_data.get('code_currency')
        if not re.match(r'^[A-Z]{3}$', code_currency):
            raise forms.ValidationError('Kode mata uang harus terdiri dari 3 huruf kapital sesuai ISO 4217.')
        return code_currency

    def save(self, commit=True):
        user = super().save(commit=False)
        if commit:
            user.save()
            UserDanantaro.objects.create(
                user=user,
                name=self.cleaned_data['name'],
                birth_date=self.cleaned_data['birth_date'],
                phone=self.cleaned_data['phone'],
                url_blog=self.cleaned_data['url_blog'],
                description=self.cleaned_data['description'],
                no_rekening=self.cleaned_data['no_rekening'],
                credit_score=self.cleaned_data['credit_score'],
                no_card=self.cleaned_data['no_card'],
                employee_status=self.cleaned_data['employee_status'],
                code_currency=self.cleaned_data['code_currency'],
            )
        return user

class LoginForm(AuthenticationForm):
    class Meta:
        model = User
        fields = [
            'username',
            'password',
        ]
        widgets = {
            'username': forms.TextInput(attrs={'placeholder': 'Masukkan username'}),
            'password': forms.PasswordInput(attrs={'placeholder': 'Masukkan password'}),
        }